{+--------------------------------------------------------------------------+
 | Component: TGifImage & TGif
 | Created: 4/12/96 11:52:35 PM
 | Author: Jeff Kinzer
 | Copyright 1996, all rights reserved.
 | Description: Displays / Converts GIF Files
 | Version: 2.0
 | Revision: 4.2
 | History:
 |    * Borland Pascal      -  GifUtl.pas     Sean Wenzel       ( CompuServe: 71736,1245 )
 |    * Created TGifImage   -  GifImg.pas     Jeff Kinzer       ( CompuServe: 102413,3557 )
 |    * Created TGif        -  Gif.pas        Jeff Kinzer
 | Unsupported GIF Features:
 |    * Misc. Extension Blocks:
 |        + Graphic Control extension
 |        + Comment extension
 |        + Plain text extension
 |        + Application extension
 | Note: I'm not sure what will happen if these
 |       blocks are encountered but it'll be interesting
 +--------------------------------------------------------------------------+}

unit GifImg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Menus, Gif;

type
  TGifImage = class(TGraphicControl)
  private
    FGifWritten: Boolean;
    FBorder: Boolean;
    FOnChange: TNotifyEvent;
    FFilename: TFilename;
    FGif: TGif;
    FOnStatusChange: TStatusChangeEvent;
    function GetFilename: TFilename;
    procedure SetFilename(newValue: TFilename);
    procedure SetGif(newValue: TGif);
    function GetBorder: Boolean;
    procedure SetBorder(newValue: Boolean);
  protected
    procedure Paint; override;
    procedure TriggerStatusChangeEvent(Sender: TObject; Status: TGifStatus); virtual;
    procedure TriggerChangeEvent; virtual;
    procedure GifChanged(Sender: TObject);
    procedure GifStatusChanged(Sender: TObject; Status: TGifStatus);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    property Canvas;
  published
    property Gif: TGif read FGif write SetGif;
    property Align;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStatusChange: TStatusChangeEvent read FOnStatusChange write FOnStatusChange;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property Border: Boolean read GetBorder write SetBorder default True;
  end;

implementation

{ TGifImage }

procedure TGifImage.Loaded;
begin
  inherited Loaded;
{  if not (FGifWritten and FGif.Empty) then
    SetGif(FGif);}
  Repaint;
end;

destructor TGifImage.Destroy;
begin
  if Assigned(FGif) then
    FGif.Free;
  inherited Destroy;
end;  { Destroy }

function TGifImage.GetBorder: Boolean;
begin
  Result := FBorder;
end;  { GetBorder }

procedure TGifImage.SetBorder(NewValue: Boolean);
begin
  if FBorder <> NewValue then
  begin
    FBorder := NewValue;
    Repaint;
  end;
end;  { SetBorder }

procedure TGifImage.GifStatusChanged(Sender: TObject; Status: TGifStatus);
begin
  if Assigned(FOnStatusChange) then
    FOnStatusChange(Self, Status);
end;  { GifStatusChanged }

procedure TGifImage.GifChanged(Sender: TObject);
begin
  Repaint;
  TriggerChangeEvent;
end;  { GifChanged }

procedure TGifImage.TriggerChangeEvent;
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;  { TriggerChangeEvent }

function TGifImage.GetFilename: TFilename;
begin
  Result := FFilename;
end;  {GetFilename}

procedure TGifImage.SetFilename(NewValue: TFilename);
begin
  if FFilename <> NewValue then
  begin
    FFilename := NewValue;
    FGif.LoadFromFile(FFilename);
    Repaint;
  end;
end;  {SetFilename}

procedure TGifImage.SetGif(NewValue: TGif);
begin
  if FGif <> NewValue then
  begin
    if Assigned(FGif) then
      FGif.Free;
    FGif := TGif.Create;
    FGif.Assign(NewValue);
    Repaint;
    FGifWritten := True;
  end;
end;  {SetGif}

procedure TGifImage.Paint;
var
  Dest: TRect;
  AColor: TColor;
  Offset: Word;
begin
  AColor := Color;
  with inherited Canvas do
  begin
    Pen.Color := clBlack;
    if FBorder then
      Pen.Style := psSolid
    else
      Pen.Style := psClear;
    Brush.Style := bsSolid;
    Brush.Color := AColor;
    Rectangle(0, 0, Width, Height);
  end;
  WordBool(Offset) := FBorder;
  Dest := Rect(Offset, Offset, FGif.FBitmap.Width + Offset, FGif.FBitmap.Height + Offset);
  with inherited Canvas do
    StretchDraw(Dest, FGif.FBitmap);
end;

procedure TGifImage.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
end;

procedure TGifImage.TriggerStatusChangeEvent(Sender: TObject; Status: TGifStatus);
begin
  if Assigned(FOnStatusChange) then
    FOnStatusChange(Self, Status);
end;  {TriggerStatusChangeEvent}

constructor TGifImage.Create(AOwner: TComponent);
begin
  FGifWritten := False;
  inherited Create(AOwner);
  Color := clWindow;
  FGif := TGif.Create;
  with FGif do
  begin
    OnChange := GifChanged;
    OnStatusChange := GifStatusChanged;
  end;
  Height := 105;
  Width := 105;
  FBorder := True;
end;

end.

